﻿B4J=true
Group=Default Group
ModulesStructureVersion=1
Type=Class
Version=7.31
@EndOfDesignText@
#DesignerProperty: Key: BarColor, DisplayName: Bar Color, FieldType: Color, DefaultValue: 0xFFB0B0B0, Description: Bar color
#DesignerProperty: Key: MenuItemColor, DisplayName: Menu Item Color, FieldType: Color, DefaultValue: 0xFF000000, Description: Menu Item color
#DesignerProperty: Key: FABPosition, DisplayName: FAB Position, FieldType: String, DefaultValue: CENTER, List: CENTER|LEFT|RIGHT
#DesignerProperty: Key: FABVisible, DisplayName: FAB visible, FieldType: Boolean, DefaultValue: True
#DesignerProperty: Key: FABInsideBar, DisplayName: FAB inside Bar, FieldType: Boolean, DefaultValue: False

#Event: FABClick
#Event: MenuItemClick(Tag As String)

Sub Class_Globals
	Private mEventName As String 'ignore
	Private mCallBack As Object 'ignore
	Private mBase As B4XView 'ignore
	Private xui As XUI 'ignore
	
	Private Bar As B4XView
	Private BarCanvas As B4XCanvas
	Private mBarColor As Int
	Private mMenuButtonColor As Int
	Private mMenuButtonFont As B4XFont
	
	Private FABBase As B4XView
	Public FAB As MDFastActionButton
	Private mFABPosition As String = "CENTER"
	Private mFABInsideBar As Boolean = False
	Private mFABVisible As Boolean = True
	Private mFABSize As Int = 56dip
	Private mFABMargin As Int = 10dip
	
	Public LeftMenuButtons As List
	Public RightMenuButtons As List
End Sub

Public Sub Initialize (Callback As Object, EventName As String)
	mEventName = EventName
	mCallBack = Callback
	
	LeftMenuButtons.Initialize
	RightMenuButtons.Initialize
	
	Bar = xui.CreatePanel("Bar")
	Bar.Color = xui.Color_Transparent
	#if b4i
	BarCanvas.Initialize(Bar)
	#end if
	mBarColor = xui.Color_LightGray
	mMenuButtonColor = xui.Color_White
	mMenuButtonFont = xui.CreateMaterialIcons(24)
	
	FABBase = xui.CreatePanel("")
	FAB.Initialize(Me,"FAB")
	
End Sub

'Base type must be Object
Public Sub DesignerCreateView (Base As Object, Lbl As Label, Props As Map)
	mBase = Base
	mBarColor = xui.PaintOrColorToColor(Props.Get("BarColor")) 'Example of getting a color value from Props
	mMenuButtonColor = xui.PaintOrColorToColor(Props.Get("MenuItemColor"))
	mFABPosition = Props.Get("FABPosition")
	mFABInsideBar = Props.Get("FABInsideBar")
	mFABVisible = Props.Get("FABVisible")
	
	DrawView
End Sub

Private Sub Base_Resize (Width As Double, Height As Double)
	DrawView
End Sub

Public Sub RuntimeCreateView(Base As B4XView)
	mBase = Base
	DrawView
End Sub

Public Sub Resize
	DrawView
End Sub

#Region View Drawing
Private Sub DrawView
	mBase.RemoveAllViews
	DrawBar
	If mFABVisible Then DrawFAB
	DrawMenu
End Sub

Private Sub DrawBar
	Dim BarHeight As Int = mBase.Height-(mFABSize/2)
	Dim BarTop As Int = mBase.Height-BarHeight
	mBase.AddView(Bar,0,BarTop,mBase.Width,BarHeight)
	#if b4a
	BarCanvas.Initialize(Bar)
	#end if
	#if b4i
	If mFABInsideBar Then
		Bar.Color = xui.Color_Transparent
	Else
		Bar.Color = mBarColor
	End If
	#End If
	BarCanvas.Resize(Bar.Width,Bar.Height)
	BarCanvas.ClearRect(BarCanvas.TargetRect)
	BarCanvas.DrawRect(BarCanvas.TargetRect,mBarColor,True,0)
	
	Select mFABPosition
		Case "LEFT"
			FABBase.Left = 16dip
		Case "CENTER"
			FABBase.Left = (mBase.Width/2)-(mFABSize/2)
		Case "RIGHT"
			FABBase.Left = (mBase.Width-20dip)-mFABSize
		Case Else
			FABBase.Left = (mBase.Width/2)-(mFABSize/2)
	End Select
	
	If mFABInsideBar = False Or mFABVisible = False Then Return
	
	'We draw the "half-circle" where we place the FAB
	Dim ButtonSpaceRect As B4XRect
	Dim Width As Double = mFABSize + mFABMargin
	Dim Height As Double = mFABSize + mFABMargin
	Dim Left As Double = FABBase.Left - (mFABMargin/2)
	Dim Top As Double =  - Width/2
	ButtonSpaceRect.Initialize(Left,Top,Left+Width,Top+Height)
	Dim ButtonSpace As B4XPath
	ButtonSpace.InitializeOval(ButtonSpaceRect)
	BarCanvas.ClipPath(ButtonSpace)
	BarCanvas.ClearRect(ButtonSpaceRect)
	BarCanvas.RemoveClip
	BarCanvas.Invalidate
	
	'We round the LEFT + RIGHT Corners of the half-circle (touch points of circle and bar)
	Dim LeftArcRect As B4XRect
	LeftArcRect.Initialize(ButtonSpaceRect.Left-mFABMargin,0,ButtonSpaceRect.Left+2dip,8dip)
	BarCanvas.ClearRect(LeftArcRect)
	Dim LeftArc As B4XPath
	Dim LARC As B4XRect
	LARC.Initialize(LeftArcRect.Left-mFABMargin,0,LeftArcRect.Right,28dip)
	LeftArc.InitializeOval(LARC)
	BarCanvas.ClipPath(LeftArc)
	BarCanvas.DrawRect(BarCanvas.TargetRect,mBarColor,True,0)
	BarCanvas.RemoveClip
	
	Dim RightArcRect As B4XRect
	RightArcRect.Initialize(ButtonSpaceRect.Right-2dip,0,ButtonSpaceRect.Right+mFABMargin,8dip)
	BarCanvas.ClearRect(RightArcRect)
	Dim RightArc As B4XPath
	Dim RARC As B4XRect
	RARC.Initialize(RightArcRect.Left,0,RightArcRect.Right+mFABMargin,28dip)
	RightArc.InitializeOval(RARC)
	BarCanvas.ClipPath(RightArc)
	BarCanvas.DrawRect(BarCanvas.TargetRect,mBarColor,True,0)
	BarCanvas.RemoveClip
	
	BarCanvas.Invalidate
End Sub

Private Sub DrawFAB
	Dim FABTop As Int = 0
	
	mBase.AddView(FABBase,FABBase.Left,FABTop,mFABSize,mFABSize)
	
	FAB.DrawShadow = Not(mFABInsideBar)
	FAB.RuntimeCreateView(FABBase)
End Sub

Private Sub DrawMenu
	Log("drawing menu")
	
	If Not(mFABPosition = "LEFT") Then
		DrawLeftMenu(Bar.Height)
	End If
	If Not(mFABPosition = "RIGHT") Then
		DrawRightMenu(Bar.Height)
	End If
End Sub

Private Sub DrawLeftMenu(ItemSize As Int)
	If LeftMenuButtons.Size = 0 Then Return
	
	Dim cIndex As Int = 0
	Do While ItemSize*(cIndex+1) < FABBase.Left
		If cIndex = LeftMenuButtons.Size Then Exit
		
		Dim MI As MDMenuItem = LeftMenuButtons.Get(cIndex)
		Dim MenuItem As B4XView = createMenuButton(MI)
		Bar.AddView(MenuItem,cIndex*ItemSize,0,ItemSize,ItemSize)
		cIndex = cIndex + 1
	Loop
End Sub

Private Sub DrawRightMenu(ItemSize As Int)
	If RightMenuButtons.Size = 0 Then Return
	
	Dim cIndex As Int = 0
	'Dim minY As Int = FABBase.Left + FABBase.Width
	Do While Bar.Width - (ItemSize*(cIndex+1)) > FABBase.Left + FABBase.Width
		If cIndex = RightMenuButtons.Size Then Exit
		
		Dim MI As MDMenuItem = RightMenuButtons.Get(cIndex)
		Dim MenuItem As B4XView = createMenuButton(MI)
		Bar.AddView(MenuItem,Bar.Width - (ItemSize*(cIndex+1)),0,ItemSize,ItemSize)
		cIndex = cIndex + 1
	Loop
End Sub

Private Sub createMenuButton(Item As MDMenuItem) As B4XView
	Dim lbl As Label
	lbl.Initialize("MenuItem")
	Dim xlbl As B4XView = lbl
	xlbl.TextColor = mMenuButtonColor
	xlbl.Font = mMenuButtonFont
	xlbl.SetTextAlignment("CENTER","CENTER")
	xlbl.Text = Item.Icon
	xlbl.Tag = Item.Tag
	
	Return xlbl
End Sub
#End Region

Sub FAB_Click
	CallSub(mCallBack,mEventName & "_FABClick")
End Sub

Sub MenuItem_Click
	Dim v As B4XView = Sender
	Dim Tag As String = v.Tag
	CallSub2(mCallBack,mEventName & "_MenuItemClick",Tag)
End Sub

Public Sub applyChanges
	DrawView
End Sub

'ONLY CALL AFTER THE VIEW HAS BEEN DESIGNER/RUNTIME CREATED
Public Sub setBarColor(Color As Int)
	mBarColor = Color
	applyChanges
End Sub
'ONLY CALL AFTER THE VIEW HAS BEEN DESIGNER/RUNTIME CREATED
Public Sub setMenuButtonColor(Color As Int)
	mMenuButtonColor = Color
	applyChanges
End Sub
'ONLY CALL AFTER THE VIEW HAS BEEN DESIGNER/RUNTIME CREATED
Public Sub setMenuButtonFont(NewFont As B4XFont)
	mMenuButtonFont = NewFont
	applyChanges
End Sub

'Position Values: CENTER,LEFT,RIGHT
'ONLY CALL AFTER THE VIEW HAS BEEN DESIGNER/RUNTIME CREATED
Public Sub setFABPosition(Position As String)
	mFABPosition = Position
	applyChanges
End Sub
'ONLY CALL AFTER THE VIEW HAS BEEN DESIGNER/RUNTIME CREATED
Public Sub setFABInsideBar(Enabled As Boolean)
	mFABInsideBar = Enabled
	applyChanges
End Sub
'ONLY CALL AFTER THE VIEW HAS BEEN DESIGNER/RUNTIME CREATED
Public Sub setFABVisible(Visible As Boolean)
	mFABVisible = Visible
	applyChanges
End Sub
Public Sub getisFABVisible As Boolean
	Return mFABVisible
End Sub
'ONLY CALL AFTER THE VIEW HAS BEEN DESIGNER/RUNTIME CREATED
Public Sub setFABSize(Size As Int)
	mFABSize = Size
	applyChanges
End Sub
'ONLY CALL AFTER THE VIEW HAS BEEN DESIGNER/RUNTIME CREATED
Public Sub setFABMargin(Margin As Int)
	mFABMargin = Margin
	applyChanges
End Sub
