﻿B4A=true
Group=Default Group
ModulesStructureVersion=1
Type=Class
Version=9
@EndOfDesignText@
'B4i Implementation of MaterialDesign Bottom Navigation Drawer. 
'Version 0.4
#Event: ItemClick (Tag As String)

Sub Class_Globals
	Private xui As XUI
	
	Private mCallBack As Object
	Private mEventName As String
	Private mTarget As B4XView 'The Target Panel, usually a Pages' RootPanel
	
	Public OverlayPanel As B4XView 'The Panel that is used as Overlay & removes the users eye focus from page content
	Public MenuPanel As B4XView 'The Basic Panel of the Menu (Items)
	
	Private MenuItems As List
	
	Public IconLabelProperties As B4XView 'Used to define the Styles of the Icon
	Public TitleLabelProperties As B4XView 'Used to define the Styles of the Text
	
	Public AnimationDuration As Int = 200
	Public isVisible As Boolean = False
End Sub

'Initializes the object. You can add parameters to this method if needed.
Public Sub Initialize(CallBack As Object, EventName As String, Target As B4XView)
	mCallBack = CallBack
	mEventName = EventName
	mTarget = Target
	
	MenuItems.Initialize
	
	'We set the default Panel Style Properties
	OverlayPanel= xui.CreatePanel("Overlay")
	OverlayPanel.Color = Colors.ARGB(200,0,0,0)
	MenuPanel = xui.CreatePanel("")
	MenuPanel.Color = Colors.White
	
	'We set the default Icon Style Properties
	IconLabelProperties = createLabel("")
	IconLabelProperties.TextColor = Colors.RGB(120,120,120)
	IconLabelProperties.SetTextAlignment("CENTER","CENTER")
	IconLabelProperties.Font = xui.CreateMaterialIcons(28)
	
	'We set the default Title Style Properties
	TitleLabelProperties = createLabel("")
	TitleLabelProperties.TextColor = Colors.RGB(20,20,20)
	TitleLabelProperties.SetTextAlignment("CENTER","LEFT")
	TitleLabelProperties.Font = xui.CreateDefaultBoldFont(17)
End Sub

Private Sub createLabel(EventName As String) As B4XView
	Dim lbl As Label
	lbl.Initialize(EventName)
	#if b4a
	lbl.SingleLine = True
	#end if
	#if b4i
	lbl.Multiline = False
	#End If
	Return lbl
End Sub

Public Sub addItem(Item As MDMenuItem)
	MenuItems.Add(Item)
	createMenuPanel
End Sub

Public Sub createItem(Title As String, Icon As String, Tag As String) As MDMenuItem
	Dim itm As MDMenuItem
	itm.Initialize
	itm.Name = Title
	itm.Icon = Icon
	itm.Tag = Tag
	Return itm
End Sub

Public Sub Resize
	createMenuPanel
End Sub

'We redesign the Menu Panel each time an item is added so we dont have to create it after an interaction / while showing
Private Sub createMenuPanel
	MenuPanel.RemoveAllViews
	MenuPanel.SetLayoutAnimated(0,0,0,mTarget.Width,24dip + MenuItems.Size*56dip)
	
	Dim YPos As Int = 12dip
	For Each MenuItem As MDMenuItem In MenuItems
		Dim ItemPanel As Panel
		ItemPanel.Initialize("Item")
		ItemPanel.Color = MenuPanel.Color
		#if b4a
		ItemPanel.SetLayoutAnimated(0,0,0,mTarget.Width,56dip)
		#end if
		#if b4i
		ItemPanel.SetLayoutAnimated(0,1,0,0,mTarget.Width,56dip)
		#End If
		ItemPanel.Tag = MenuItem
		
		Dim lblIcon As Label = createLabel("")
		lblIcon.Initialize("")
		lblIcon.TextColor = IconLabelProperties.TextColor
		#if b4i
		lblIcon.TextAlignment = toNativeLabel(IconLabelProperties).TextAlignment
		lblIcon.Font = IconLabelProperties.Font.ToNativeFont
		#end if
		#if b4a
		lblIcon.Gravity = toNativeLabel(IconLabelProperties).Gravity
		lblIcon.Typeface = IconLabelProperties.Font.ToNativeFont
		lblIcon.TextSize = IconLabelProperties.TextSize
		#end if
		lblIcon.Text = MenuItem.Icon
		
		Dim lblTitle As Label
		lblTitle.Initialize("")
		lblTitle.TextColor = TitleLabelProperties.TextColor
		#if b4i
		lblTitle.TextAlignment = toNativeLabel(TitleLabelProperties).TextAlignment
		lblTitle.Font = TitleLabelProperties.Font.ToNativeFont
		#end if
		#if b4a
		lblTitle.Gravity = toNativeLabel(TitleLabelProperties).Gravity
		lblTitle.Typeface = TitleLabelProperties.Font.ToNativeFont
		lblTitle.TextSize = TitleLabelProperties.TextSize
		#end if
		lblTitle.Text = MenuItem.Name
		
		ItemPanel.AddView(lblIcon,0,0,ItemPanel.Height+12dip,ItemPanel.Height)
		ItemPanel.AddView(lblTitle,lblIcon.Left+lblIcon.Width,0,ItemPanel.Width-52dip,ItemPanel.Height)
		
		MenuPanel.AddView(ItemPanel,0,YPos,ItemPanel.Width,ItemPanel.Height)
		
		YPos = YPos + ItemPanel.Height
	Next
End Sub

Private Sub toNativeLabel(lbl As B4XView) As Label
	Dim nlbl As Label = lbl
	Return nlbl
End Sub

Private Sub Item_Click
	Dim v As View = Sender
	Dim Item As MDMenuItem = v.Tag
	CallSub2(mCallBack,mEventName & "_ItemClick",Item.Tag)
End Sub

Public Sub showMenu
	OverlayPanel.Visible = False
	mTarget.AddView(OverlayPanel,0,0,mTarget.Width,mTarget.Height)
	OverlayPanel.SetVisibleAnimated(Max(0,AnimationDuration-100),True)
	
	mTarget.AddView(MenuPanel,0,mTarget.Height,MenuPanel.Width,MenuPanel.Height)
	MenuPanel.SetLayoutAnimated(AnimationDuration,0,mTarget.Height-MenuPanel.Height,MenuPanel.Width,MenuPanel.Height)
	isVisible = True
End Sub

Public Sub hideMenu
	OverlayPanel.SetVisibleAnimated(Max(0,AnimationDuration-100),False)
	MenuPanel.SetLayoutAnimated(AnimationDuration,0,mTarget.Height,MenuPanel.Width,MenuPanel.Height)
	
	Sleep(AnimationDuration)
	OverlayPanel.RemoveViewFromParent
	MenuPanel.RemoveViewFromParent
	isVisible = False
End Sub

Private Sub Overlay_Click
	hideMenu
End Sub